@@ -1,8 +1,8 @@
 /*
   +----------------------------------------------------------------------+
-  | PHP Version 5                                                        |
+  | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2015 The PHP Group                                |
+  | Copyright (c) 1997-2017 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -25,6 +25,9 @@
 extern zend_module_entry pdo_sqlite_module_entry;
 #define phpext_pdo_sqlite_ptr &pdo_sqlite_module_entry
 
+#include "php_version.h"
+#define PHP_PDO_SQLITE_VERSION PHP_VERSION
+
 #ifdef ZTS
 #include "TSRM.h"
 #endif
@@ -35,9 +38,9 @@ PHP_RINIT_FUNCTION(pdo_sqlite);
 PHP_RSHUTDOWN_FUNCTION(pdo_sqlite);
 PHP_MINFO_FUNCTION(pdo_sqlite);
 
-/* 
+/*
   	Declare any global variables you may need between the BEGIN
-	and END macros here:     
+	and END macros here:
 
 ZEND_BEGIN_MODULE_GLOBALS(pdo_sqlite)
 	long  global_value;
@@ -45,16 +48,6 @@ ZEND_BEGIN_MODULE_GLOBALS(pdo_sqlite)
 ZEND_END_MODULE_GLOBALS(pdo_sqlite)
 */
 
-/* In every utility function you add that needs to use variables 
-   in php_pdo_sqlite_globals, call TSRMLS_FETCH(); after declaring other 
-   variables used by that function, or better yet, pass in TSRMLS_CC
-   after the last function argument and declare your utility function
-   with TSRMLS_DC after the last declared argument.  Always refer to
-   the globals in your function as PDO_SQLITE_G(variable).  You are 
-   encouraged to rename these macros something shorter, see
-   examples in any other php module directory.
-*/
-
 #ifdef ZTS
 #define PDO_SQLITE_G(v) TSRMG(pdo_sqlite_globals_id, zend_pdo_sqlite_globals *, v)
 #else
