@@ -110,7 +110,7 @@ PHPAPI int php_url_encode_hash_ex(HashTable *ht, smart_str *formstr,
 				char *ekey;
 				size_t ekey_len;
 				/* Is an integer key */
-				ekey_len = spprintf(&ekey, 0, "%pd", idx);
+				ekey_len = spprintf(&ekey, 0, ZEND_LONG_FMT, idx);
 				newprefix_len = key_prefix_len + num_prefix_len + ekey_len + key_suffix_len + 3 /* %5B */;
 				newprefix = emalloc(newprefix_len + 1);
 				p = newprefix;
